package SystemUpdater;
require CMONOS::SetupAssistant::Tools::SetupEnv;
@ISA = qw(SetupEnv);
use File::Path;
use Fcntl qw(:DEFAULT :flock);
eval 'use ' . $env::dbm_file . ';';
use strict;
use utf8;
######################################################################
# システム・アップデート モジュール
# SystemUpdater.pm ver1.0.1
# Copyright (C) 1999-2023 CMONOS Co. Ltd.
#
# support@cmonos.jp
# http://cmonos.jp/
# 最終更新日：2023-08-31
######################################################################
# 実行
sub run ($) {
    my $this = shift;
    $this->get_setup_env;

    my $download_dir = 'https://cmonos.jp/download/CMONOS_JP_1x';
    if ( $] >= 5.008001 && $cgi::cmonos_ver >= 20110905 ) {
        $download_dir
            = ($default::beta_enabled)
            ? 'https://cmonos.jp/download/CMONOS_JP_beta'
            : 'https://cmonos.jp/download/CMONOS_JP_2x';
    }

    require CMONOS::CGI_Lib::HTTP_Client;
    my $res = HTTP_Client->new->request(
        'GET' => $download_dir . '/latest_version.txt' );
    my $latest_version = ( $res->is_success ) ? $res->content : 20110214;
    $latest_version = 20110905
        if !$latest_version || $latest_version =~ /[^0-9]/s;
    unless ( eval 'use Archive::Tar;Archive::Tar->has_zlib_support;' ) {
        $this->jump_menu('tools');
        $this->print_page_header('tools');
        my $dl_url
            = $download_dir
            . (
            ( $^X =~ /local/ ) ? '/CMONOS_JP_local.tgz' : '/CMONOS_JP.tgz' );
        print '<h1>システム・アップデート</h1>';
        if ( $latest_version > $cgi::cmonos_ver ) {
            print
                '<p class="warning">より新しいバージョンがみつかりました。</p>';
        }
        else {
            print '<p>お使いのシステムは最新版です。</p>';
        }
        print
            '<p>Archive::Tar モジュールがみつからないため、この機能は利用できません。</p>';
        print '<p>「<a href="' . $dl_url
            . '">CMONOS.JP フルセット版</a>」をダウンロードし、同梱の HowToUpdate.html にしたがってアップデートしてください。</p>';
        $this->print_page_footer('tools');
        return 1;
    }
    my $temp_dir    = $env::clipboard . '/system_updater';
    my $status_file = $temp_dir . '/status';
    rmtree $temp_dir if -d $temp_dir && -M $temp_dir > 1;
    my $status = $this->read_file( $status_file, 1 ) || 0;
    if ($status) {
        if ( $status >= 5 || ( stat($status_file) )[9] <= time - 900 ) {
            $main::system_update = 1;
        }
        else {
            undef $main::system_update;
        }
    }

    #パーミッション調整
    if ($main::system_update) {
        my $error;
        if ( $status < 2 ) {
            $this->set_dir($temp_dir);
            $this->write_file( $status_file, 1, 1 );
            my $res = HTTP_Client->new->request(
                'GET' => $download_dir . '/CMONOS_JP_Updater.tgz',
                {}, '', '', '', $temp_dir . '/CMONOS_JP_Updater.tgz'
            );
            if ( $res->is_success && -f $temp_dir . '/CMONOS_JP_Updater.tgz' )
            {
                $this->write_file( $status_file, 2, 1 );
            }
            else {
                $error = 1;
                push(
                    @{ $this->{'dialog'} },
                    'CMONOS.JP 最新版のダウンロードに失敗しました。'
                );
            }
        }
        if ( !$error && $status < 3 ) {
            $this->write_file( $status_file, 3, 1 );
            my $tar = Archive::Tar->new();
            $Archive::Tar::WARN = 0;    # 警告を出力しない
            $tar->setcwd($temp_dir);
            $tar->read( $temp_dir . '/CMONOS_JP_Updater.tgz', 1 );
            $tar->extract();
            if ( -f $temp_dir . '/CMONOS_JP_Updater/Updater_Combo.pm' ) {
                $this->write_file( $status_file, 4, 1 );
            }
            else {
                $error = 1;
                push(
                    @{ $this->{'dialog'} },
                    'CMONOS.JP 最新版の解凍に失敗しました。'
                );
            }
        }
        if ( !$error && $status < 5 ) {

#unless ($this->directory_copy($temp_dir . '/CMONOS_JP_Updater/CMONOS',$default::shared_cgi_data_root . '/lib/CMONOS',1)) {
            unless (
                rename(
                    $default::shared_cgi_data_root . '/lib/CMONOS',
                    $temp_dir . '/CMONOS_JP_Updater/CMONOS.old'
                )
                && rename(
                    $temp_dir . '/CMONOS_JP_Updater/CMONOS',
                    $default::shared_cgi_data_root . '/lib/CMONOS'
                )
                )
            {
                push(
                    @{ $this->{'dialog'} },
                    'CMONOS.JP ライブラリの更新に失敗しました。'
                );
                $error = 1;
            }

#unless ($this->directory_copy($temp_dir . '/CMONOS_JP_Updater/lib',$env::template_dir . '/site/default/htdocs/lib',1)) {
            unless (
                rename(
                    $env::template_dir . '/site/default/htdocs/lib',
                    $temp_dir . '/CMONOS_JP_Updater/lib.old'
                )
                && rename(
                    $temp_dir . '/CMONOS_JP_Updater/lib',
                    $env::template_dir . '/site/default/htdocs/lib'
                )
                )
            {
                push(
                    @{ $this->{'dialog'} },
                    'デフォルトの JavaScript/CSS ライブラリの更新に失敗しました。'
                );
                $error = 1;
            }
            $this->system_chmod();
            $this->write_file( $status_file, 5, 1 ) if !$error;
        }
        if ( !$error ) {
            $this->write_file( $status_file, 6, 1 );
            push( @setting::form, 'system_update' );
            require $temp_dir . '/CMONOS_JP_Updater/Updater_Combo.pm';
            my $updater = new Common('....');
            if (  !$main::ver
                && %setting::general
                && $setting::general{'ver'}
                && $latest_version > $setting::general{'ver'} )
            {
                $main::ver       = $setting::general{'ver'};
                $main::updating  = 1;
                $cgi::cmonos_ver = $latest_version;
            }
            $updater->run;
            exit;
        }
        rmtree $temp_dir;
    }

    #ページヘッダ
    $this->jump_menu('tools');
    $this->print_page_header('tools');

    print '<h1>システム・アップデート</h1>';
    if ( $latest_version > $cgi::cmonos_ver ) {
        print
            '<p class="warning">より新しいバージョンがみつかりました。</p>';
    }
    else {
        print '<p>お使いのシステムは最新版です。</p>';
    }

    if ($status) {
        my %status_msg = (
            1 => 'CMONOS.JP 最新版をダウンロードしています...',
            2 =>
                'CMONOS.JP 最新版のダウンロードが完了しました...',
            3 => 'CMONOS.JP 最新版を解凍しています...',
            4 => 'CMONOS.JP 最新版の解凍が完了しました...',
            5 =>
                'CMONOS.JP ライブラリファイルを更新しました...',
            6 => 'CMONOS.JP 最新版にアップデートしています...',
        );
        print '<p>';
        print( $status_msg{$status} || 'Error!' );
        print '</p>';

        #フォーム開始
    }
    else {
        print
            '<p class="warning">システムをアップデートすると「~/cgi-data/template/site/default/htdocs/lib」が最新のファイルで上書きされます。ご注意ください。</p>';
        print $this->form_start(
            'key'    => 'tools',
            'name'   => 'input',
            'method' => 'post',
            'onsubmit' =>
                'return confirm(\'この操作は取り消せません。本当にシステムをアップデートしますか？\') &amp;&amp; cmonos.form.item(this).isNotSubmitted();',
            'required' => \@setting::form
        );
        print '<input type="hidden" name="system_update" value="1" />';
        print '<fieldset class="delete">';
        print
            '<input type="submit" value="最新版をダウンロードしてシステムをアップデートする" />';
        print '</fieldset>';
        print '</form>';
    }
    print
        '<p><a href="http://cmonos.jp/download/history.html" target="_blank">&raquo;バージョン履歴</a> <a href="http://cmonos.jp/download/CMONOS_JP_manual.zip">&raquo;マニュアルをダウンロードする</a></p>';

    #ページフッタ
    $this->print_page_footer('tools');
}

sub clean_up ($) {
    my $this = shift;
    rmtree $env::clipboard . '/system_updater';
}

sub system_chmod ($$) {
    my ( $this, $rw ) = @_;
    local ( $SystemUpdater::fmod, $SystemUpdater::dmod )
        = ($rw) ? ( 0666, 0777 )
        : ($env::ownerpower) ? ( 0600, 0700 )
        :                      ( $env::file_mode, $env::dir_mode );

    use File::Find;
    find(
        sub {
            if ( -f $_ ) {
                chmod( $SystemUpdater::fmod, $_ );
            }
            elsif ( -d $_ ) {
                chmod( $SystemUpdater::dmod, $_ );
            }
        },
        $default::shared_cgi_data_root . '/lib/CMONOS'
    );
}

######################################################################
# クラス登録
#sub import {
BEGIN {
    if (   $env::ownerpower
        && ( !defined $env::home_dir_id || $env::home_dir_id eq '' )
        && !$env::multi_domain )
    {
        no warnings qw(once);

        #クラス登録
        Common::class_register( 's201', 'SystemUpdater' );

        #メニュー登録
        $setting::jump_menu{'s201'} = 'システム・アップデート';
    }
}

1;
